@@ -40,17 +40,6 @@ class Grain(ArgumentParser):
     """
 
     def __init__(self, polyaxon_exp=None, *args, **kwargs):
-        """Initialise Grain object using CLI arguments
-
-        Parameters
-        ----------
-        polyaxon_exp : a polyaxon experiment to instantiate
-                Grain object
-        *args        : a non keyworded arguments' list
-        **kwargs     : a keyworded arguments' list
-
-        *args and **kwargs can be of any length
-        """
         super(Grain, self).__init__(*args, **kwargs)
         self.polyaxon_exp = polyaxon_exp
 
@@ -70,7 +59,7 @@ class Grain(ArgumentParser):
                           help='patch size for training process')
 
         self.add_argument('--resolution',
-                          type=int,
+                          type=float,
                           required=True,
                           help='resolution of training process tensor')
 
@@ -124,7 +113,8 @@ class Grain(ArgumentParser):
                 '--resolution',
                 str(metadata['resolution'])
             ])
-            logging.debug("Enter parse_args_from_json routine")
+
+            logging.debug("Exit parse_args_from_json routine")
             return args
 
     def load_model(self, model_cls, **kwargs):
