@@ -21,8 +21,10 @@ class FocalLoss(nn.Module):
 
     Parameters
     ----------
-    args : type
-        Description of parameter `args`.
+    alpha : float
+        alpha
+    gamma : int
+        gamma
 
     References
     ----------
@@ -30,18 +32,10 @@ class FocalLoss(nn.Module):
 
     """
 
-    def __init__(self, args):
-        """Initialise loss module.
-
-        Parameters
-        ----------
-        args : type
-            Description of parameter `args`.
-
-        """
+    def __init__(self, alpha, gamma):
         super(FocalLoss, self).__init__()
-        self.alpha = args.alpha
-        self.gamma = args.gamma
+        self.alpha = alpha
+        self.gamma = gamma
 
     def forward(self, predicted, target):
         """Compute loss between :attr:`predicted` and :attr:`target`.
