@@ -32,35 +32,24 @@ class SplineLoss(nn.Module):
 
     Parameters
     ----------
-    args : list
-        arguments list.
+    patch_size: int
+        patch size
+    w : int
+        weight
+    lambda_p : int
+        lambda
+
 
     References
     ----------
     https://sci-hub.do/https://ieeexplore.ieee.org/document/8953484
     """
 
-    def __init__(self, args):
-        """Initialise loss module.
-
-        Parameters
-        ----------
-        args : list
-            arguments list.
-
-        """
+    def __init__(self, patch_size, w=1, lambda_p=1):
         super(SplineLoss, self).__init__()
-        self.patch_size = args.input_shape[2]
-        self.gpu = args.gpu
-
-        if hasattr(args, 'w'):
-            self.w = args.w
-        else:
-            self.w = 1
-        if hasattr(args, 'lambda_p'):
-            self.lambda_p = args.lambda_p
-        else:
-            self.lambda_p = 1
+        self.patch_size = patch_size
+        self.w = w
+        self.lambda_p = lambda_p
         self.epsilon = 1e-8
 
     def forward(self, predicted, target):
@@ -100,9 +89,9 @@ class SplineLoss(nn.Module):
 
         c_1 = torch.ones((self.patch_size, self.patch_size))
         c_2 = torch.zeros((self.patch_size, self.patch_size))
-        if self.gpu > -1:
-            c_1 = c_1.cuda(self.gpu)
-            c_2 = c_2.cuda(self.gpu)
+
+        c_1 = c_1.to(predicted.device)
+        c_2 = c_2.to(predicted.device)
 
         region_in = torch.abs(
             torch.sum(
