@@ -1,4 +1,5 @@
 import numpy as np
+from sklearn import metrics
 import torch
 import logging
 
@@ -18,6 +19,7 @@ from sklearn.metrics import (f1_score, precision_score, recall_score)
 from .mIoU_and_SeK import mIoU, SeK
 from .multilabel import emr, hamming
 from .accuracy import Acc
+from copy import deepcopy
 
 
 metrics_map = {
@@ -46,11 +48,38 @@ metrics_map = {
     'acc': Acc
 }
 
-def set_metric(key, metric):
+metrics_argmax_required = [
+    'dc',
+    'jc',
+    'hd',
+    'asd',
+    'assd',
+    'prec',
+    'recall',
+    'sensi',
+    'speci',
+    'ravd',
+    'tpr',
+    'tnr',
+    'ppv',
+    'mc-prec',
+    'mc-recall',
+    'mc-f1-score',
+    'ml-prec',
+    'ml-recall',
+    'ml-emr',
+    'ml-hamming',
+    'mIoU',
+    'SeK',
+    'acc'
+]
+
+def set_metric(key, metric, argmax_required=False):
     """Allows: 
 
-    1.Addition of a new metric to metrics map
-    2.Modification of existing metric definitions in metrics map
+    * Addition of a new metric to metrics map
+
+    * Modification of existing metric definitions in metrics map
 
     Parameters
     ----------
@@ -61,6 +90,8 @@ def set_metric(key, metric):
     """
     logging.debug("Enter set_metrics routine")
     metrics_map[key] = metric
+    if argmax_required:
+        metrics_argmax_required.append(key)
     logging.debug("Exit set_metrics routine")
 
 
@@ -112,9 +143,9 @@ class Metrics():
         self.metric_funcs = {}
         self.metrics = {}
         self.distributed = distributed
-
         self.multilabel = False
         self.num_classes = num_classes
+
         self.initialize_metrics()
 
     def initialize_metrics(self):
@@ -162,6 +193,8 @@ class Metrics():
 
             self.metric_funcs[metstr] = metrics_map[metstr]
             self.metrics[metstr] = []
+
+        self.tboard = None
     
     def reset(self):
         """Reset metrics map.
@@ -188,15 +221,17 @@ class Metrics():
         logging.debug("Inside _transform_tensor routine")
         if isinstance(tensor, int):
             return tensor
-        if isinstance(tensor, torch.Tensor):
+        elif isinstance(tensor, torch.Tensor):
             if tensor.is_cuda:
                 return tensor.data.cpu().numpy()
             else:
                 return tensor.data.numpy()
-        if isinstance(tensor, np.ndarray):
+        elif isinstance(tensor, np.ndarray):
             return tensor
-        if isinstance(tensor, list):
+        elif isinstance(tensor, list):
             return np.asarray(tensor)
+        else:
+            return tensor
 
     def _argmax_or_thresholding(self, tensor):
         """Performs argmax or thresholding on input tensor.
@@ -214,7 +249,7 @@ class Metrics():
         """
         logging.debug("Enter _argmax_or_thresholding routine")
         if type(tensor) != torch.Tensor:
-            raise TypeError("Input should be a torch tensor!")
+            raise TypeError("Input should be a torch tensor! but got "+str(type(tensor)))
 
         if len(tensor.size()) >= 4:
             if tensor.size(1) == 1:
@@ -236,6 +271,26 @@ class Metrics():
         logging.debug("Exit _argmax_or_thresholding routine")
         return tensor
 
+    def computeMetricsOut(self,k,predicted_,target_):
+        metrics_out = None
+
+        if 'mc' in k:
+            metrics_out = self.metric_funcs[k](predicted_.flatten(),
+                                                        target_.flatten(),
+                                                        average='micro')
+        elif 'ml' in k:
+            metrics_out = self.metric_funcs[k](predicted_,
+                                                        target_,
+                                                        average='samples')
+        elif 'mIoU' in k or 'SeK' in k:
+            metrics_out = self.metric_funcs[k](predicted_,
+                                                        target_,
+                                                        num_classes=self.num_classes)
+        else:
+            metrics_out = self.metric_funcs[k](predicted_,
+                                                        target_)
+        return metrics_out
+
     def compute(self, predicted, target, loss):
         """Compute loss between target and predicted tensors.
 
@@ -249,40 +304,42 @@ class Metrics():
             loss tensor.
 
         """
+        predicted = predicted.detach().cpu()
+        target = target.detach().cpu()
+
         self.metrics['loss'].append(self._transform_tensor(loss))
 
         if len(self.metric_strings) > 0:
-            predicted = self._argmax_or_thresholding(predicted)
 
             for k in self.metric_funcs.keys():
-                predicted = self._transform_tensor(predicted)
-                target = self._transform_tensor(target)
+                predicted_,target_ = deepcopy(predicted), deepcopy(target)
+                if k in metrics_argmax_required:
+                    predicted_ = self._argmax_or_thresholding(predicted_)
+                
 
-                if 'mc' in k:
-                    self.metrics[k].append(self.metric_funcs[k](predicted.flatten(),
-                                                                target.flatten(),
-                                                                average='micro'))
-                elif 'ml' in k:
-                    self.metrics[k].append(self.metric_funcs[k](predicted,
-                                                                target,
-                                                                average='samples'))
-                elif 'mIoU' in k or 'SeK' in k:
-                    self.metrics[k].append(self.metric_funcs[k](predicted,
-                                                                target,
-                                                                num_classes=self.num_classes))
-                else:
-                    self.metrics[k].append(self.metric_funcs[k](predicted,
-                                                                target))
+                try:
+                    metrics_out = self.computeMetricsOut(k,predicted_,target_)
+                except:
+                    predicted_ = self._transform_tensor(predicted_)
+                    target_ = self._transform_tensor(target_)
+                    metrics_out = self.computeMetricsOut(k,predicted_,target_)
 
+                del predicted_,target_
+                metrics_out = self._transform_tensor(metrics_out)
+                self.metrics[k].append(metrics_out)
                 logging.info("{}={}".format(k, self.metrics[k][-1]))
 
-    def crunch_it(self, epoch):
+    def plotTensorboard(self,mean_metrics,step):
+        for key in mean_metrics.keys():
+            self.tboard.add_scalar(f"{self.phase}/{key}",mean_metrics[key],step)
+
+    def crunch_it(self, step):
         """Crunch metrics to obtain mean_metrics map.
 
         Parameters
         ----------
-        epoch : int
-            model epoch.
+        step : int
+            training step.
 
         Returns
         -------
@@ -310,9 +367,10 @@ class Metrics():
         if self.distributed:
             dist.barrier()
             for k in mean_metrics.keys():
-                dist.all_reduce(torch.tensor(mean_metrics[k]).cuda(),
+                _ = torch.tensor(mean_metrics[k]).cuda()
+                dist.all_reduce(_,
                                 op=ReduceOp.SUM)
-                mean_metrics[k] = mean_metrics[k].item() / float(dist.get_world_size())
+                mean_metrics[k] = _.cpu().item()/ float(dist.get_world_size())
             dist.barrier()
 
         # logging
@@ -324,8 +382,15 @@ class Metrics():
         if self.polyaxon_exp:
             if self.distributed:
                 if dist.get_rank() == 0:
-                    self.polyaxon_exp.log_metrics(step=epoch, **mean_metrics)
+                    self.polyaxon_exp.log_metrics(step=step, **mean_metrics)
+                    if self.tboard:
+                        self.plotTensorboard(mean_metrics=mean_metrics,step=step)
                 dist.barrier()
             else:
-                self.polyaxon_exp.log_metrics(step=epoch, **mean_metrics)
+                self.polyaxon_exp.log_metrics(step=step, **mean_metrics)
+                if self.tboard:
+                    self.plotTensorboard(mean_metrics=mean_metrics,step=step)
+        else:
+            if self.tboard:
+                self.plotTensorboard(mean_metrics=mean_metrics,step=step)
         return mean_metrics
