@@ -2,12 +2,12 @@ import numpy as np
 import logging
 
 
-def emr(predicted, target, **kwargs):
+def emr(predicted, target, average):
     logging.debug("Evaluating Exact-Match-Ratio(EMR)")
     return np.all(target == predicted, axis=1).mean()
 
 
-def hamming(predicted, target, **kwargs):
+def hamming(predicted, target, average):
     logging.debug("Evaluating Hamming loss")
     tmp = 0
     for i in range(target.shape[0]):
