@@ -1,15 +1,19 @@
 from .scheduler import get_scheduler
 from .optimizer import get_optimizer
+from torch.autograd import Variable
+from torch.utils.data.distributed import DistributedSampler
+from phobos.loss import get_loss
+from phobos.metrics.metrics import Metrics
+from torch.utils.tensorboard import SummaryWriter
 
 import torch
 import logging
-
+import os
 import torch.distributed as dist
+import tqdm
+import math
 
-from torch.autograd import Variable
 
-from phobos.loss import get_loss
-from phobos.metrics.metrics import Metrics
 
 
 class Runner():
@@ -29,6 +33,8 @@ class Runner():
         dataloader to load validation dataset.
     distributed : boolean
         flag to create runner in distributed mode
+    val_node : int
+        validation step node if args.distributed=True, -1 for distributed val step
     metrics : str / phobos.metrics.metrics.Metrics
         metrics string
     optimizer : str / torch.optim
@@ -64,14 +70,16 @@ class Runner():
         scheduler instance
     distributed : boolean
         flag to create runner in distributed mode
+    distributed_val : boolean
+        DDP: distributed validation step if True, val_step on master node otherwise.
     train_metrics : phobos.metrics.metrics.Metrics
         training metrics instance
     val_metrics : phobos.metrics.metrics.Metrics
         validation metrics instance
     polyaxon_exp : poyaxon.tracking.Run
         polyaxon experiment.
-    epoch : int
-        current epoch
+    iteration : int
+        current iteration
     """
 
     def __init__(self,
@@ -80,15 +88,21 @@ class Runner():
                  criterion,
                  train_loader,
                  val_loader,
-                 distributed,
                  metrics,
                  optimizer,
+                 distributed=False,
+                 max_iters:int = 0,
+                 distributed_val=True,
+                 mode = 'epoch',
+                 val_frequency: int = -1,
+                 tensorboard_logging:bool = True,
                  num_classes=None,
                  loss_args=None,
                  scheduler=None,
                  optimizer_args=None,
                  scheduler_args=None,
                  polyaxon_exp=None):
+                 
         self.polyaxon_exp = polyaxon_exp
         self.model = model
         self.device = device
@@ -96,16 +110,39 @@ class Runner():
         self.train_loader = train_loader
         self.val_loader = val_loader
 
+        self.mode = mode
+        self.max_iters = max_iters
+
+        assert mode == 'epoch' or mode == 'batch', 'Enter the correct mode epoch/batch'
+
         self.criterion = self.get_runner_criterion(criterion, loss_args)
         self.optimizer = self.get_runner_optimizer(optimizer, optimizer_args, model)        
         self.scheduler = self.get_runner_scheduler(scheduler, scheduler_args, self.optimizer)
 
         self.distributed = False
         if distributed:
-            self.set_distributed_params()
+            self.set_distributed_params(distributed_val=distributed_val)
 
         self.train_metrics, self.val_metrics = self.get_runner_metrics(metrics, num_classes)
-        self.epoch = 0
+
+        if self.distributed and distributed_val == False:
+            assert type(self.val_loader.sampler) !=  DistributedSampler, "For single node val_step, val_loader should not have distributed sampler"
+            self.val_metrics.distributed = False
+        
+        self.val_frequency = val_frequency
+        self.tboard = None
+        
+        if tensorboard_logging:
+            self.tensorboard_logging = True
+            if polyaxon_exp:
+                tensorboard_path = os.path.join(polyaxon_exp.get_artifacts_path(),'outputs/tensorboard')
+            else:
+                tensorboard_path = os.path.join(os.curdir,'outputs/tensorboard')
+            if not os.path.exists(tensorboard_path):
+                os.makedirs(tensorboard_path)
+            self.tboard = SummaryWriter(log_dir=tensorboard_path)
+            self.train_metrics.tboard = self.tboard
+            self.val_metrics.tboard = self.tboard
 
     @staticmethod
     def distributed():
@@ -114,7 +151,7 @@ class Runner():
         """
         dist.init_process_group(backend='nccl')
 
-    def set_distributed_params(self):
+    def set_distributed_params(self, distributed_val):
         """Set up distributed params: world size, rank, and distributed or not.
 
         """
@@ -122,6 +159,7 @@ class Runner():
         self.rank = dist.get_rank()
         logging.info(f"{self.rank} / {self.world_size}")
         self.distributed = True
+        self.distributed_val = distributed_val
 
     def get_runner_metrics(self, metrics, num_classes=None):
         if type(metrics) == list:
@@ -162,18 +200,6 @@ class Runner():
         else:
             return None
 
-    def set_epoch_metrics(self):
-        """Sets epoch metrics and increments epoc count
-            at the end of every epoch.
-
-        """
-        logging.debug("Enter set_epoch_metrics routine")
-        self.train_metrics.reset()
-        self.val_metrics.reset()
-        self.epoch += 1
-        logging.warning("Epoch : {}".format(self.epoch))
-        logging.debug("Exit set_epoch_metrics routine")
-
     def tensorize_batch(self, input_tensor, label_tensor):
         """Tensorize batch of input images and labels,
             and move them to gpu.
@@ -262,35 +288,14 @@ class Runner():
 
         return prediction_tensor, loss
 
-    def train_model(self):
-        """Trains model.
-
-        Returns
-        -------
-        dict
-            dictionary of training metrics.
-
-        """
-        logging.debug("Enter train_model routine")
-        self.model.train()
-
-        for input_tensor, label_tensor in self.train_loader:
-            input_tensor, label_tensor = self.tensorize_batch(
-                input_tensor, label_tensor)
-
-            prediction_tensor, loss = self.train_forward_backward(
-                input_tensor, label_tensor)
-            self.train_metrics.compute(prediction_tensor, label_tensor, loss)
-
-            # clear batch variables from memory
-            del input_tensor, label_tensor
-        logging.debug("Exit train_model routine")
-
-        return self.train_metrics.crunch_it(self.epoch)
-
-    def eval_model(self):
+    def eval_model(self,step):
         """Evaluates model.
 
+        Parameters
+        ----------
+        step : int
+            training step.
+
         Returns
         -------
         dict
@@ -311,10 +316,94 @@ class Runner():
                 # clear batch variables from memory
                 del input_tensor, label_tensor
 
-        metrics = self.val_metrics.crunch_it(self.epoch)
+
+        metrics = self.val_metrics.crunch_it(step)
 
         if self.scheduler:
             self.scheduler.step(metrics['val_loss'])
         logging.debug("Exit eval_model routine")
 
         return metrics
+
+    def getContextIterationValue(self,x):
+        val = x//self.train_loader.batch_size
+        if self.train_loader.drop_last:
+            val += math.ceil(x/self.train_loader.batch_size - x//self.train_loader.batch_size)
+        return val
+
+    def logResults(self,step):
+        val_metrics = None
+        if self.distributed:
+            if self.distributed_val or dist.get_rank() == 0:
+                val_metrics = self.eval_model(step)
+                self.val_metrics.reset()
+                dist.barrier()
+        else:
+            val_metrics = self.eval_model(step)
+            self.val_metrics.reset()
+        return val_metrics
+        
+    def condRun(self,iteration,epoch):
+        if self.mode == 'epoch' and epoch < self.max_iters:
+            return True
+        elif self.mode == 'batch' and iteration < self.max_iters:
+            return True
+        return False
+
+    def trainer(self):
+        """Trains model.
+
+        Returns
+        -------
+        dict
+            dictionary of training metrics.
+
+        """
+        logging.debug("Enter train_model generator")
+
+        train_metrics, val_metrics = None,None
+        iteration,epoch = 0,0
+
+        while  self.condRun(iteration,epoch):
+            epoch += 1
+            for input_tensor, label_tensor in self.train_loader:
+                iteration += 1
+                # batch train
+                self.model.train()
+
+                input_tensor, label_tensor = self.tensorize_batch(
+                    input_tensor, label_tensor)
+
+                prediction_tensor, loss = self.train_forward_backward(
+                    input_tensor, label_tensor)
+                self.train_metrics.compute(prediction_tensor, label_tensor, loss)
+
+                del input_tensor, label_tensor
+                
+                if self.mode == 'batch':
+                    train_metrics = self.train_metrics.crunch_it(iteration)
+                    self.train_metrics.reset()
+                    if self.distributed:
+                        dist.barrier()
+                    #end batch train
+                    logging.info(f"{iteration}/{self.max_iters} completed!")
+                    if self.val_frequency != -1 and iteration % self.val_frequency == 0:
+                        val_metrics = self.logResults(iteration)
+                        yield iteration,train_metrics,val_metrics
+
+                if self.mode == "batch" and not self.condRun(iteration,epoch):
+                    break
+                    
+            if self.mode == 'epoch':
+                train_metrics = self.train_metrics.crunch_it(epoch)
+                self.train_metrics.reset()
+                logging.info(f"{epoch}/{self.max_iters} completed!")
+                if self.distributed:
+                    dist.barrier()
+                val_metrics = self.logResults(epoch)
+                yield epoch, train_metrics, val_metrics
+            elif self.mode == 'batch' and self.val_frequency == -1:
+                val_metrics = self.logResults(iteration)
+                yield epoch,train_metrics,val_metrics
+                
+        logging.debug("Exit train_model routine")
\ No newline at end of file
